
/* ---------------------------------------------------------------------------------------------
 *  ArdweenyShield TMP36 Sensor interface using POV bargraph, and Servo indicator needle
 * ---------------------------------------------------------------------------------------------
 *   
 *  Measure the temperature from the TMP36 on Analog3, and turn it into an LED barscale
 *  on the 8 SMD LED pads on the ArdweenyShield belly. Now with Servo goodness!
 * 
 *  Code borrowed/cribbed/copied/modified/defiled from the fine folks at:
 *    Oomlout (for their TMP36 tutorial at http://tinyurl.com/c89tvd)
 *    Adafruit (jeez, there's lots of TMP36 examples there)
 *    Arduino team (Dave Mellis' Smoothing routing)
 *
 *  Solarbotics.com
 *  Version 0.3
 */

//----------------------------------------------------------------------------
//clear the comment "//" on the below "#define" to get console feedback 
//#define DBUG         


//----------------------------------------------------------------------------
//Following is the setups for the smoothing routine:

//Higher the number, the smoother the reading, the SLOWER the response. 
// Yes, 100 readings seems about right to fix the Arduino ADC jumpiness.
const int numReadings = 100;

// the readings from the analog input
int readings[numReadings];      

// the index of the current reading
int index = 0;                  

// the running total
int total = 0;                  

// the average
int average = 0;                

//the analog pin the TMP36's Vout (sense) pin is connected to. 
int TempInputPin = A3;

//----------------------------------------------------------------------------
//Following is the bargraph setup

// The size of the LED bargraph array
const int LEDCount = 8;

//The array determining the steps for the LEDs to light up in
const float TempSteps [] = {
  18,20,22,24,26,28,30,32}; 

//These be the pins connected to the LEDs (useful for reversing the up/down of the scale)
int LEDpins [] = {
  0,1,2,3,4,5,6,7};

// The top LED to illuminate in the string (0 to 7)
int LEDTop;

//----------------------------------------------------------------------------
//Calculations setup

// The thermal value to be tested from TempSteps Array
float Therm;

// Variable to read/calculate the temperature
float Temperature;

// For storing the raw value for Debugging information
int TemperatureRaw;

//----------------------------------------------------------------------------
//Servo Setup
//Load up servo know-how into Ardweeny
#include <Servo.h>

// create servo object to control a servo 
Servo TemperatureServo;

/* Adjust these values for your servo and setup, if necessary
   Sweep is supposed to be 15 to 165 degrees, but servo splines made these numbers
   shove back a bit to properly align with the scale we printed
*/

int servoPin     =  9;       // Digital I/O line D9 for servo motor header
int minPulse     =  19;      // minimum servo ANGLE position (max temperature for clockwise scale)
int maxPulse     =  169;     // maximum servo ANGLE position (min temperature for clockwise scale)
int servoVal;



//----------------------------------------------------------------------------
//Get ready to party...

/*
 * setup() - Start your Ardweeny doing this stuff ONCE on reset / power-up
 */
void setup()
{
  //IF the debug is turned on, we start communications  
#ifdef DBUG           
  Serial.begin(9600);  //with the copmuter, (yes, the copmuter). To see the data, open the serial monitor 
  //with the last button beneath the file bar (looks like a box with an antennae)
#endif                       
  
  // initialize the SMD bellypads 0 thru 7 as outputs, and test the LEDs for 1/2 second
  for (int i=0; i<LEDCount; i++) {
    pinMode(i, OUTPUT);
  }
  
//The following is a rapid blink "HEY - I've REBOOTED" (for servo over-current draw detection)
{ for (int x=0; x < 10; x++)
  digitalWrite(13, HIGH);   // set the LED on
  delay(10);              // wait for a second
  digitalWrite(13, LOW);    // set the LED off
  delay(10);              // wait for a second
} 
  // initialize all the data smoothing readings to 0: 
  for (int thisReading = 0; thisReading < numReadings; thisReading++)
    readings[thisReading] = 0;

  // Initialize Servo
  TemperatureServo.attach(servoPin);  // attaches the servo on pin "servoPin" to the servo object 
  //Test the servo sweep
  TemperatureServo.write(0);  //Show the maximum pointer scale position
  delay (3000);
  TemperatureServo.write(minPulse);  //Show the maximum pointer scale position
  delay (1000);
  TemperatureServo.write(maxPulse); //Show the minimum pointer scale position
  delay (1000);
  


  
  delay (500);
}

//----------------------------------------------------------------------------
//Start to Party! Woohoo!

void loop() 
{

  //Quick - take the multiple samples of the temperatures sensor!
  //-------------------------------------------------------------
  for (int quicksamples=0; quicksamples < numReadings; quicksamples++)
  {
    //Let's get the reading, and apply it to the smoothing function.
    // Subtract the last reading:
    total= total - readings[index];

    // Read from the sensor:  
    readings[index] = analogRead(TempInputPin);

    //Save reading for echoing out to error console
    TemperatureRaw = readings[index];

    // Add the reading to the total:
    total= total + readings[index];

    // Advance to the next position in the array:  
    index = index + 1;                    

    // if we're at the end of the array...
    if (index >= numReadings)              
      // ...wrap around to the beginning: 
      index = 0;                           

    // calculate the average:
    average = total / numReadings; 
  }

  //-------------------------------------------------------------
  //Do some calculations

    //Convert reading to voltage
  Temperature = average * 5.0 / 1024;

  //Convert to degrees Celcius. 10mV per degree (with 0.5V offset)
  Temperature = (Temperature - 0.5) * 100;

#ifdef DBUG
  // Let's see the smoothing array list!
  /*Serial.println("Smoothing Array:");
   for (int arraylist = 0; arraylist < numReadings; arraylist++)
   {
   Serial.print (readings[arraylist], DEC);
   Serial.print (", ");
   }
   Serial.println ();  
   */
  Serial.print("average: ");
  Serial.println(average, DEC);
  Serial.print("TemperatureRaw: ");
  Serial.println(TemperatureRaw, DEC);
  Serial.print("Temperature: ");
  Serial.println(Temperature, 3);
  Serial.println();
#endif

  //Read Bargraph array, from highest value going down. Set "Therm" to value stored at array index "BGIndex"
  for (int BGIndex=LEDCount-1; BGIndex>=0; BGIndex--)
  {
    Therm=TempSteps[BGIndex];

    // Check steps if DBUG is set up. If so, echo the following to the terminal
    // Let's set the top LED of the temperature bargraph.
    // If "Therm" is greater than "Temperature", remember the top LED position as "LEDTop", 
    // and set "BGIndex" to ("ArrayMax" +1) to exit loop quicker that before
    if (Therm < Temperature) 
    {
      LEDTop=BGIndex;
      BGIndex=0;
    }
#ifdef DBUG
    Serial.print("Therm: ");
    Serial.println(Therm);
    Serial.print("BGIndex ");
    Serial.println(BGIndex);
#endif
  }

  // Turn off all the LEDs in the bargraph
  PORTD = B00000000;


  //Turn on all the LEDs from 0 to the index value that matched the measured temperature
  for (int LEDs=0; LEDs <= LEDTop; LEDs++)
  {
    digitalWrite(LEDs, HIGH);
  }

  //Map servo range from 0 to 50 degrees C
  //The map function is neat. It remaps a number from one range to another, in this case
  //being from a temperatures scale of the -50 to +50 degrees C (a reasonable range for Canada!)
  //set for the travel of the servo "maxPulse" and "minPulse" defined at the start.
  //Use the extremes of the "TempSteps" array (index values TempSteps[0] and [7], if you 
  //want the servo range to match the same range on the bargraph.
  servoVal = map(Temperature, -50, 50, maxPulse, minPulse); 
  TemperatureServo.write(servoVal);


  //waiting a second
  delay(1000);                                  

}









